
// Portions of code from
// https://github.com/MellongLau/AMMethod2Implement
// MIT license

#import "P4XcodeHelper.h"
#import <AppKit/AppKit.h>

@implementation P4XcodeHelper

+ (NSURL*)currentProjectURL
{
    for (NSDocument* currentDoc in [NSApp orderedDocuments])
    {
        @try
        {
            NSURL* url = [[[currentDoc valueForKeyPath:@"_workspace.representingFilePath.fileURL"]
                           URLByDeletingLastPathComponent]
                          filePathURL];
            
            if (url)
            {
                return url;
            }
        }
        
        @catch (NSException* exception)
        {
            NSLog(@"Exception raised while searching for workspace URL: %@", exception);
        }
    }
    
    return nil;
}

/*
 https://github.com/MellongLau/AMMethod2Implement/blob/master/AMMethod2Implement/AMXcodeHelper.m
 */
+ (id)currentEditor
{
    NSWindow* window = [NSApp keyWindow];
    NSWindowController *windowController = [window windowController];
    if (windowController == nil)
    {
        return nil;
    }
    
    if ([windowController isKindOfClass:NSClassFromString(@"IDEWorkspaceWindowController")])
    {
        IDEWorkspaceWindowController *workspaceController = (IDEWorkspaceWindowController *)windowController;
        IDEEditorArea *editorArea                         = [workspaceController editorArea];
        IDEEditorContext *editorContext                   = [editorArea lastActiveEditorContext];
        return [editorContext editor];
    }
    return nil;
}

+ (NSObject *)currentWorkspaceDocument
{
    NSWindowController *currentWindowController = [[NSApp keyWindow] windowController];
    id document = [currentWindowController document];
    if (currentWindowController && [document isKindOfClass:NSClassFromString(@"IDEWorkspaceDocument")])
    {
        return (NSObject *)document;
    }
    return nil;
}

+ (IDESourceCodeDocument *)currentSourceCodeDocument
{
    id currentEditor = [P4XcodeHelper currentEditor];
    if (currentEditor == nil)
    {
        return nil;
    }
    
    if ([currentEditor isKindOfClass:NSClassFromString(@"IDESourceCodeEditor")])
    {
        IDESourceCodeEditor *editor = [P4XcodeHelper currentEditor];
        return editor.sourceCodeDocument;
    }
    
    if ([currentEditor isKindOfClass:NSClassFromString(@"IDESourceCodeComparisonEditor")])
    {
        IDESourceCodeComparisonEditor *editor = currentEditor;
        if ([[editor primaryDocument]
             isKindOfClass:NSClassFromString(@"IDESourceCodeDocument")])
        {
            IDESourceCodeDocument *document =
            (IDESourceCodeDocument *)editor.primaryDocument;
            return document;
        }
    }
    if ([currentEditor respondsToSelector:@selector(document)]) {
        IDESourceCodeDocument *document = (IDESourceCodeDocument *)[currentEditor document];
        
        return document;
    }
    
#if (0) // not yet implemented
    if ([currentEditor isKindOfClass:NSClassFromString(@"IDEQuickLookEditor")])
    {
        IDESourceCodeEditor *editor = [P4XcodeHelper currentEditor];
        return editor.sourceCodeDocument;
    }
#endif
#if defined(_DEBUG)
    if([currentEditor respondsToSelector:@selector(className)])
    {
        NSLog(@"Class type %@\n", [currentEditor className]);
    }
#endif
    
    return nil;
}

+ (NSURL*)currentSourceCodeDocumentURL
{
    NSURL* docURL = nil;
    IDESourceCodeDocument * doc = [P4XcodeHelper currentSourceCodeDocument];
    if (doc != nil)
    {
        BOOL isDirectory;
        docURL = [doc fileURL];
        if ([[NSFileManager defaultManager] fileExistsAtPath:[docURL path] isDirectory:&isDirectory]) {
            if (isDirectory) {
                NSString *path = [docURL path];
                path = [path stringByAppendingPathComponent:@"project.pbxproj"];
                if ([[NSFileManager defaultManager] fileExistsAtPath:[docURL path]]) {
                    docURL = [NSURL fileURLWithPath:path];
                }
            }
        }
        
    }
    
    return docURL;
}

@end

//@implementation IDESourceCodeDocument
//- (DVTSourceTextStorage *)textStorage
//{
//    return nil;
//}
//
//- (NSUndoManager *)undoManager
//{
//    return nil;
//}
//
//@end